//******************************************************************************
//                       MAIN PROGRAM PIC16F1789                               *
//                       ***********************                               *
//   This file is used to set the port pins of the  PIC16F1789                 *
//                                                                             *
//******************************************************************************
//                                                                             *
//   Filename:      PIC16F1789_MAIN.c                                          *
//   Date:          19/10/2014                                                 *
//   File Version:  Ver 1.0                                                    *
//                                                                             *
//   Author:        Simon O'Farrell                                            *
//                                                                             *
//                                                                             *
//******************************************************************************
//                                                                             *
//   Files required:                                                           *
//                   <xc.h>                                                    *
//                   <stdint.h>                                                *
//                                                                             *
//                                                                             *
//******************************************************************************
//                                                                             *
//                                                                             *
//                               PIC16F1789 PINOUT                             *
//                               *****************                             *
//                                  __________                                 *
//                                 |          |                                *
//                 +5v ------!MCLR |1       40| RB7(PGD)-- (ICSP)              *
//    (ADC NTC MOSFET) ------I-RA0 |2       39| RB6(PGC)-- (ICSP)              *
//  (ADC CURRENT LOAD) ------|-RA1 |3       38| RB5-|----- (Unused)            *
//  (ADC VOLTAGE LOAD) ------|-RA2 |4       37| RB4-|----- (Unused)            *
//            (Unused) ------|-RA3 |5       36| RB3------I-ENCODER - (Current) *
//            (Unused) ------|-RA4 |6       35| RB2------I-ENCODER - (Current) *
//            (Unused) ------|-RA5 |7       34| RB1-O----- (Unused)            *
//            (Unused) ------|-RE0 |8       33| RB0/INT--  (ENC - SW)          *
//            (Unused) ------|-RE1 |9       32| VDD  ----- +5v                 *
//            (Unused) ------|-RE2 |10      31| VSS  ----- GND                 *
//                 +5v ------  VDD |11      30| RD7-O----- LCD_DB7(LCD pin 14) *
//                 GND ------  VSS |12      29| RD6-O----- LCD_DB6(LCD pin 13) *
//            (Unused) ------ OSC1 |13      28| RD5-O----- LCD_DB5(LCD pin 12) *
//            (Unused) ------ OSC2 |14      27| RD4-O----- LCD_DB4(LCD pin 11) *
//         (SPI Clock) ------  RC0 |15      26| RC7-|----- (Unused)            *
//          (SPI Data) ------|-RC1 |16      25| RC6-|----- (Unused)            *
//          (SPI SYNC) ------|-RC2 |17      24| RC5-|----- (Unused)            *
//            (Unused) ------|-RC3 |18      23| RC4-|----- (Unused)            *
//  LCD_rs (LCD pin 4) ------O-RD0 |19      22| RD3 |----- (Unused)            *
//  LCD_rw (LCD pin 5) ------O-RD1 |20      21| RD2-O----- LCD_e - (LCD pin 6) *
//                                 |__________|                                *
//                                                                             *
//******************************************************************************

#include <xc.h>

#include "PIC16F1789_System_Manager.h"
#include "PIC16F1789_Pin_Manager.h"


// --> I N T E R R U P T   S E R V I C E    R O U T I N E <--
//     **************************************************
void interrupt isr(void)
{
    const int8_t encStates[] = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0};

            
    if (INTCONbits.T0IE == 1 && INTCONbits.T0IF == 1)
    {
        // Service TIMER0 interrupt source (Overflows every 4.096ms).

        timer0Flag = 1;                // Set the TIMER0 flag.
        
        INTCONbits.T0IF = 0;           // Clear TMR0 INTERRUPT flag.        
    }

    else if (PIE1bits.ADIE && PIR1bits.ADIF == 1)
    {
        ADCFlag = 1;                   // Set the ADC flag.
        
        PIR1bits.ADIF = 0;             // Clear ADC INTERRUPT flag.        
    }
    
    else if(INTCONbits.IOCIE == 1 && 
            (IOCBFbits.IOCBF2 == 1 || IOCBFbits.IOCBF3 == 1)) 
    {
        //--> Servicing external interrupts <--


        pinState <<= 2;                    // Remember previous state.

        pinState |= (PIN_2 << 1) | (PIN_1);// Shift PIN1 left 1, add PIN2 and
                                           //  OR the combination with pinState.

        // Using the combination of previous and current state as an index 
        //  to the state table encStates.
        encoderDirection += encStates[(pinState & 0x0F)];

        IOCBF = 0;                         // Reset interrupt for PORTB.
    }

}// --> I N T E R R U P T   S E R V I C E    R O U T I N E   E N D <--