//******************************************************************************
//                  SYSTEM MANAGER FILE FOR PIC16F1789                         *
//                  **********************************                         *
//   This file is used to set the port pins of the  PIC16F1789                 *
//                                                                             *
//******************************************************************************
//                                                                             *
//   Filename:      PIC16F1789_System_Manager.c                                *
//   Date:          Created on 21 June 2015, 8:30 PM                                                 *
//   File Version:  Ver 1.0                                                    *
//                                                                             *
//   Author:        Simon O'Farrell                                            *
//                                                                             *
//                                                                             *
//******************************************************************************
//                                                                             *
//   Files required:                                                           *
//                   <xc.h>                                                    *
//                   <stdint.h>                                                *
//                                                                             *
//                                                                             *
//******************************************************************************
//                                                                             *
//                                                                             *
//                               PIC16F1789 PINOUT                             *
//                               *****************                             *
//                                  __________                                 *
//                                 |          |                                *
//                 +5v ------!MCLR |1       40| RB7(PGD)-- (ICSP)              *
//    (ADC NTC MOSFET) ------I-RA0 |2       39| RB6(PGC)-- (ICSP)              *
//  (ADC CURRENT LOAD) ------|-RA1 |3       38| RB5-|----- (Unused)            *
//  (ADC VOLTAGE LOAD) ------|-RA2 |4       37| RB4-|----- (Unused)            *
//            (Unused) ------|-RA3 |5       36| RB3------I-ENCODER - (Current) *
//            (Unused) ------|-RA4 |6       35| RB2------I-ENCODER - (Current) *
//            (Unused) ------|-RA5 |7       34| RB1-O----- (Unused)            *
//            (Unused) ------|-RE0 |8       33| RB0/INT--  (ENC - SW)          *
//            (Unused) ------|-RE1 |9       32| VDD  ----- +5v                 *
//            (Unused) ------|-RE2 |10      31| VSS  ----- GND                 *
//                 +5v ------  VDD |11      30| RD7-O----- LCD_DB7(LCD pin 14) *
//                 GND ------  VSS |12      29| RD6-O----- LCD_DB6(LCD pin 13) *
//            (Unused) ------ OSC1 |13      28| RD5-O----- LCD_DB5(LCD pin 12) *
//            (Unused) ------ OSC2 |14      27| RD4-O----- LCD_DB4(LCD pin 11) *
//         (SPI Clock) ------  RC0 |15      26| RC7-|----- (Unused)            *
//          (SPI Data) ------|-RC1 |16      25| RC6-|----- (Unused)            *
//          (SPI SYNC) ------|-RC2 |17      24| RC5-|----- (Unused)            *
//            (Unused) ------|-RC3 |18      23| RC4-|----- (Unused)            *
//  LCD_rs (LCD pin 4) ------O-RD0 |19      22| RD3 |----- (Unused)            *
//  LCD_rw (LCD pin 5) ------O-RD1 |20      21| RD2-O----- LCD_e - (LCD pin 6) *
//                                 |__________|                                *
//                                                                             *
//******************************************************************************

#include <xc.h>
#include <stdint.h>
#include <stdio.h>
#include <stdbool.h>

#include "PIC16F1789_System_Manager.h"
#include "PIC16F1789_Pin_Manager.h"
#include "ROTARY_ENCODER_FUNCTIONS.h"

//--> C O N F I G U R A T I O N   W O R D  <--
//    *************************

// --> C O N F I G   1 <--
#pragma config FOSC = INTOSC           // Oscillator Selection bits (HS
                                       //     oscillator)

#pragma config WDTE = OFF              // Watchdog Timer Enable bit (WDT
                                       //     disabled)

#pragma config MCLRE = ON              // MCLR/VPP pin function is MCLR.

#pragma config CP = OFF                // Flash Program Memory Code
                                       //     Protection bit (Code protection
                                       //     off)

#pragma config CPD = OFF               // Data EEPROM Memory Code Protection
                                       //     bit (Data EEPROM code
                                       //     protection off)

#pragma config BOREN = ON              // Brown-out Reset Enable bit (BOR
                                       //     enabled)

#pragma config CLKOUTEN = OFF          // CLKOUT function is disabled.
                                       //     I/O or oscillator function
                                       //     on the CLKOUT pin.

#pragma config IESO = OFF              // Internal/External Switchover
                                       //     mode is disabled.

#pragma config FCMEN = OFF             // Fail-Safe Clock Monitor is disabled.

// --> C O N F I G   2 <--
#pragma config WRT = OFF               // Write protection off.

#pragma config VCAPEN = OFF            // Vcap functionality is disabled on RA6.

#pragma config PLLEN = ON             // 4x PLL Enabled.

#pragma config STVREN = ON             // Stack Overflow or Underflow will
                                       //     cause a Reset.

#pragma config BORV = LO               // Brown-out Reset Voltage (Vbor),
                                       //     low trip point selected.

#pragma config LPBOR = ON              // Low power brown-out is enabled.

#pragma config LVP = OFF               // Low-Voltage programming disabled.
 

//--> C O N F I G U R A T I O N  W O R D  E N D <--


// -->  S Y S T E M    I N I T I A L I S A T I O N  <--

void SYSTEM_Initialise(void) 
{
    OSCILLATOR_Initialise();
    //LCD_Initialise();
 //   ADC_Initialise();
//    TM0Initialise();
 //   rotaryEncoderInitialisation();
}



void OSCILLATOR_Initialise(void)
{
    OSCCONbits.SPLLEN = 1;             // Software PLL Enable bit - Enable = 1.

    OSCCONbits.IRCF3 = 1;              // We require 16MHz so IRCF = 1111.
    OSCCONbits.IRCF2 = 1;
    OSCCONbits.IRCF1 = 1;
    OSCCONbits.IRCF0 = 1;

    OSCCONbits.SCS0 = 0;              // With FOSC <2:0> = 100 (INTOSC) and
    OSCCONbits.SCS1 = 0;              //  these two turned off or 00.
}