//******************************************************************************
//                       MAIN PROGRAM PIC16F1789                               *
//                       ***********************                               *
//   This file is used to set the port pins of the  PIC16F1789                 *
//                                                                             *
//******************************************************************************
//                                                                             *
//   Filename:      PIC16F1789_MAIN.c                                          *
//   Date:          19/10/2014                                                 *
//   File Version:  Ver 1.0                                                    *
//                                                                             *
//   Author:        Simon O'Farrell                                            *
//                                                                             *
//                                                                             *
//******************************************************************************
//                                                                             *
//   Files required:                                                           *
//                   <xc.h>                                                    *
//                   <stdint.h>                                                *
//                                                                             *
//                                                                             *
//******************************************************************************
//                                                                             *
//                                                                             *
//                               PIC16F1789 PINOUT                             *
//                               *****************                             *
//                                  __________                                 *
//                                 |          |                                *
//                 +5v ------!MCLR |1       40| RB7(PGD)-- (ICSP)              *
//    (ADC NTC MOSFET) ------I-RA0 |2       39| RB6(PGC)-- (ICSP)              *
//  (ADC CURRENT LOAD) ------|-RA1 |3       38| RB5-|----- (Unused)            *
//  (ADC VOLTAGE LOAD) ------|-RA2 |4       37| RB4-|----- (Unused)            *
//            (Unused) ------|-RA3 |5       36| RB3------I-ENCODER - (Current) *
//            (Unused) ------|-RA4 |6       35| RB2------I-ENCODER - (Current) *
//            (Unused) ------|-RA5 |7       34| RB1-O----- (Unused)            *
//            (Unused) ------|-RE0 |8       33| RB0/INT--  (ENC - SW)          *
//            (Unused) ------|-RE1 |9       32| VDD  ----- +5v                 *
//            (Unused) ------|-RE2 |10      31| VSS  ----- GND                 *
//                 +5v ------  VDD |11      30| RD7-O----- LCD_DB7(LCD pin 14) *
//                 GND ------  VSS |12      29| RD6-O----- LCD_DB6(LCD pin 13) *
//            (Unused) ------ OSC1 |13      28| RD5-O----- LCD_DB5(LCD pin 12) *
//            (Unused) ------ OSC2 |14      27| RD4-O----- LCD_DB4(LCD pin 11) *
//         (SPI Clock) ------  RC0 |15      26| RC7-|----- (Unused)            *
//          (SPI Data) ------|-RC1 |16      25| RC6-|----- (Unused)            *
//          (SPI SYNC) ------|-RC2 |17      24| RC5-|----- (Unused)            *
//            (Unused) ------|-RC3 |18      23| RC4-|----- (Unused)            *
//  LCD_rs (LCD pin 4) ------O-RD0 |19      22| RD3 |----- (Unused)            *
//  LCD_rw (LCD pin 5) ------O-RD1 |20      21| RD2-O----- LCD_e - (LCD pin 6) *
//                                 |__________|                                *
//                                                                             *
//******************************************************************************

#include <xc.h>
#include <stdint.h>

#include "PIC16F1789_System_Manager.h"
#include "PIC16F1789_Pin_Manager.h"
#include "ROTARY_ENCODER_FUNCTIONS.h"

// --> C O N S T A N T S <--
#ifndef NSAMPLES

#define NSAMPLES   16

#endif

// --> G L O B A L   F L A G   V A R I A B L E S   F O R
//     T H E   I N T E R U P T   V E C T O R <--
// This is the timer flag triggered from the interrupt vector.
int8_t pinState = 0;

int8_t encoderDirection = 0;

uint8_t timer0Flag = 0;

uint8_t ADCFlag = 0;

// --> G L O B A L   V A R I A B L E S   E N D  <--


int main(void) {
    // -->  V A R I A B L E S <--
    uint16_t dutyCycle = 0; // duty cycle between 0 and 99.

    //Variables for CURRENT
    uint16_t smp_bufI[16];

    uint8_t sI = 0;

    uint16_t adc_sumI = 0;

    uint16_t adc_decI = 0;

    uint8_t channel = 0;


    //Variables for VOLTAGE
    uint16_t smp_bufV[16];

    uint8_t sV = 0;

    uint16_t adc_sumV = 0;

    static uint16_t adc_decV = 0;
    
    // --> E N D   O F   V A R I A B L E   D E C L A R A T I O N S <--


    // --> I N I T I A L I S E <--
    rotaryEncoderInitialisation();

    //writeStringXY(5, 1, "DUMMY LOAD");
    //writeStringXY(5, 2, "**********");


    __delay_ms(1000); // Delay of 1 second.

    //clearDisplay();

    //displayCurrentValues();



    // Finally, enable global INTERRUPTS.
    INTCONbits.GIE = 1; // Enable global INTERUPT.


    // --> E N D   O F   I N I T I A L I S A T I O N S <--


    // --> S T A R T   O F   M A I  N   L O O P <--
    while (1) 
    {
        if (encoderDirection > 5) 
        {
            if (dutyCycle < 100) 
            {
                dutyCycle = dutyCycle + 1;
            }
            encoderDirection = 0;

            //modifyDutyCycle(dutyCycle);

            //displayValues(dutyCycle);
        } else if (encoderDirection < -5) 
        {
            if (dutyCycle > 0) 
            {
                dutyCycle = dutyCycle - 1;
            }
            encoderDirection = 0;

            //modifyDutyCycle(dutyCycle);

            //displayValues(dutyCycle);
        }


//        if (timer0Flag) {
//            switch (channel) {
//                case 0:
//                    channel = 1;
//
//                    changeChannel(channel);
//
//                    break;
//
//                case 1:
//                    channel = 2;
//
//                    changeChannel(channel);
//
//                    break;
//
//                case 3:
//                    channel = 0;
//
//                    changeChannel(channel);
//
//                    break;
//            }
//
//            ADCON0bits.GO = 1; // Initiate new ADC conversion.
//
//            timer0Flag = 0;
//
//        }
//
//        if (ADCFlag) {
//
//            if (channel) {
//                adc_decI = movingAverage(&adc_sumI, smp_bufI, &sI);
//
//                ADCFlag = 0;
//            } else {
//                adc_decV = movingAverage(&adc_sumV, smp_bufV, &sV);
//
//                ADCFlag = 0;
//            }
//
//        }

    }
}

